<?php

namespace IPS\brilliantdiscord\Util;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

use IPS\brilliantdiscord\CdnUtil;
use IPS\brilliantdiscord\RateLimit;
use IPS\brilliantdiscord\Request;

class _Guild
{
    /**
     * Access guild data
     *
     * @param int $optionalInviteAge Maximum lifespan of optionally created invite
     * @param bool $noCache Whether to use cached data or not
     * @return array
     */
    public static function guildData($optionalInviteAge = 3600, $noCache=FALSE)
    {
        if ($noCache || !isset(\IPS\Data\Store::i()->discord_guild_data) || \IPS\Data\Store::i()->discord_guild_data['timestamp'] + 600 < time()) {
            $gid = \IPS\Settings::i()->brilliantdiscord_guild;
            RateLimit::limitHandle('guilds/{guild.id}', $gid, function ($check) use ($gid) {
                $request = new Request("guilds/$gid", ['with_counts' => 'true']);
                $request->applyDefaultHeaders();
                $request->bot();
                $response = $request->submit();
                $check($response);

                switch ($response->httpResponseCode) {
                    case 200:
                        $json = $response->decodeJson();
                        $avatar = CdnUtil::guildIcon($gid, $json['icon']);
                        if ($avatar != NULL) {
                            $avatar = (string) $avatar;
                        }
                        \IPS\Data\Store::i()->discord_guild_data = [
                            'timestamp' => time(),
                            'name' => $json['name'],
                            'letters' => static::lettersForIcon($json['name']),
                            'avatar' => $avatar,
                            'online' => $json['approximate_presence_count'],
                            'overall' => $json['approximate_member_count']
                        ];
                        break;
                    default:
                        (new UnhandledDiscordException($request, $response))->safeHandle(true, '4sbr110/1');
                }
            });
        }
        return \IPS\Data\Store::i()->discord_guild_data;
    }

    public static function lettersForIcon($name) {
        $takeNext = TRUE;
        $letters = [];
        foreach (str_split($name) as $char) {
            if ($char == ' ') {
                $takeNext = TRUE;
                continue;
            }
            if (!preg_match('/\d|[a-z]/i', $char)) {
                $letters[] = $char;
                $takeNext = TRUE;
                continue;
            }
            if ($takeNext) {
                $letters[] = $char;
                $takeNext = FALSE;
                continue;
            }
        }
        return implode('', $letters);
    }

    public static function roles($gid = NULL, $token = NULL, $identifier=NULL) {
        $gid = $gid ?: \IPS\Settings::i()->brilliantdiscord_guild;
        return RateLimit::limitHandle('guilds/{guild.id}/roles', $gid, function($check) use ($gid, $token) {
            $request = new Request("guilds/{$gid}/roles");
            $request->applyDefaultHeaders();
            $request->bot($token);
            $response = $request->submit();
            $check($response);
            if ($response->httpResponseCode != 200) {
                (new UnhandledDiscordException($request, $response))->safeHandle();
            } else {
                $roles = [];
                foreach ($response->decodeJson() as $role) {
                    if ($role['managed'] || $role['id'] == $gid) continue;
                    $roles[$role['id']] = $role['name'];
                }
                return $roles;
            }
        }, $identifier ?: \IPS\Settings::i()->brilliantdiscord_cid);
    }

    protected function dummy() {}
}