<?php

namespace IPS\brilliantdiscord\Util;

/* To prevent PHP errors (extending class does not exist) revealing path */
if (!defined('\IPS\SUITE_UNIQUE_KEY')) {
    header((isset($_SERVER['SERVER_PROTOCOL']) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0') . ' 403 Forbidden');
    exit;
}

use IPS\brilliantdiscord\Maintenance\Log;
use IPS\brilliantdiscord\Request;
use IPS\Http\Response;

class _UnhandledDiscordException extends \RuntimeException
{
    /**
     * @var Request
     */
    public $request;
    /**
     * @var Response
     */
    public $response;

    /**
     * UnhandledDiscordException constructor.
     *
     * @param Request $request
     * @param Response $response
     */
    public function __construct(Request $request, Response $response)
    {
        $this->request = $request;
        $this->response = $response;
        $endpoint = mb_substr((string)$request->getUrl(), mb_strlen(Request::API_URL));
        parent::__construct("Unexpected $response->httpResponseCode response code at /$endpoint", $response->httpResponseCode);
    }

    public function requestData()
    {
        return ['content' => $this->response->content, 'req_head' => $this->request->headers(), 'res_head' => $this->response->httpHeaders, 'req_data' => static::stripTokens($this->request->lastReqData)];
    }

    public function safeHandle($displayErrorPage = TRUE, $errorPageCode = '4SBR000/2', $errorMessage = NULL, $adminErrorMessage = NULL)
    {
        if (\IPS\IN_DEV) {
            throw $this;
        }

        Log::log($this);
        if ($displayErrorPage) {
            \IPS\Output::i()->error($errorMessage ?: 'generic_error', $errorPageCode, 500, $adminErrorMessage);
        }
    }

    public static function stripTokens($data, $key=null)
    {
        if (in_array($key, ['access_token', 'client_secret'])) {
            return '***';
        }

        $wasJson = false;
        if (is_string($data) && in_array(mb_substr($data, 0, 1), ['{', '['])) {
            $json = json_decode($data, true);
            if ($json !== null) {
                $wasJson = true;
                $data = $json;
            }
        }
        if (is_array($data)) {
            foreach ($data as $k => &$v) {
                $v = static::stripTokens($v, $k);
            }
        }

        if ($wasJson) {
            $data = json_encode($data);
        }

        return $data;
    }
}